use crate::config::{Context, WebSocketCfg};
use crate::ip::IpEvent;
use crate::utils::gethostname;

use std::net::TcpStream;
use std::sync::Arc;
use tokio::sync::mpsc::Sender;
use tokio::sync::RwLock;
use tungstenite::stream::*;
use tungstenite::*;

pub async fn websocketconnect<'a>(
    wscfg: &WebSocketCfg,
) -> Result<WebSocket<MaybeTlsStream<TcpStream>>, Error> {
    let (socket, _response) = connect(&wscfg.endpoint).expect("Can't connect");
    Ok(socket)
}

pub async fn websocketinit(
    ctx: &Arc<RwLock<Context>>,
    ipeventtx: &Sender<IpEvent>,
) -> Arc<RwLock<WebSocket<MaybeTlsStream<TcpStream>>>> {
    let ctxarc = Arc::clone(&ctx);

    let wssocket;
    let wssocketcb;
    {
        let ctx = ctxarc.read().await;
        wssocket = Arc::new(RwLock::new(
            websocketconnect(&ctx.cfg.ws.get("websocket").unwrap())
                .await
                .unwrap(),
        ));

        wssocketcb = wssocket.clone();
    }

    wslistenpubsub(wssocket, ipeventtx.clone()).await;
    return wssocketcb;
}

async fn wslistenpubsub(
    socket: Arc<RwLock<WebSocket<MaybeTlsStream<TcpStream>>>>,
    txpubsub: Sender<IpEvent>,
) {
    tokio::spawn(async move {
        loop {
            let msgs: Option<String>;
            {
                let mut socket = socket.write().await;
                msgs = match socket.read_message() {
                    Ok(s) => {
                        println!("msg: {}", s);
                        None
                    }
                    Err(e) => {
                        println!("error: {e:?}");
                        socket.close(None).unwrap();
                        return;
                    }
                };
            }
            match msgs {
                Some(ss) => {
                    let tosend: IpEvent = serde_json::from_str(ss.as_str()).unwrap();
                    if tosend.ipdata.hostname != gethostname(true)
                        || tosend.msgtype == "init".to_string()
                    {
                        txpubsub.send(tosend).await.unwrap();
                    }
                }
                None => {}
            };
        }
    });
}

pub async fn send_to_ipbl_websocket(
    wssocket: &Arc<RwLock<WebSocket<MaybeTlsStream<TcpStream>>>>,
    ip: &IpEvent,
    _ret: &mut Vec<String>,
) {
    let msg = format!("{val}", val = serde_json::to_string(&ip).unwrap());

    let wsclone = Arc::clone(wssocket);
    let mut ws = wsclone.write().await;

    match ws.write_message(Message::Text(msg)) {
        Ok(_) => {}
        Err(e) => {
            println!("{e:?}")
        }
    };
    match ws.read_message() {
        Ok(o) => {
            println!("{o}")
        }
        Err(e) => {
            println!("{e:?}")
        }
    };
}
